import os

from flask import Blueprint, current_app as app
from flask import request, jsonify, url_for, render_template
import jinja2

try:
    from jinja2 import pass_context as jinja_context
except:
    print("Warning, using outdated Flask+Jinja versions! Please upgrade Flask and Jinja soon.")
    from jinja2 import contextfilter as jinja_context

from accpy import acxml, jaya, accBaseDir

mod_apex = Blueprint('apex', __name__, static_url_path='/apex/static', static_folder='./static', template_folder='./templates')

@jinja_context
@mod_apex.app_template_filter()
def inline_js(context, js_file):
    file_path = os.path.join(mod_apex.static_folder, 'js', js_file)
    with app.open_resource(file_path, mode='r') as f:
        data = f.read()
    return data

@jinja_context
@mod_apex.app_template_filter()
def inline_css(context, css_file):
    file_path = os.path.join(mod_apex.static_folder, 'css', css_file)
    with app.open_resource(file_path, mode='r') as f:
        data = f.read()
    return data


@mod_apex.route("/_get_beam_props",methods = ["POST"])
def _get_beam_props():
    d = {'status':'Post request Received.'}
    data = request.json

    if 'path' not in data:
        d['status']='Error - path must be defined in request.'
        return jsonify(d), 400

    beam_prop_json = acxml.get_beamprops_by_path(data['path'], json=True)
    beam_prop_list = [item['@pv'] for item in beam_prop_json]
    pv_readings = jaya.get_pvs(beam_prop_list)
    epics_checks = get_epics_checks(data['path'], beam_prop_list)
    return jsonify( {'pvlist': beam_prop_list, 'pvdict': pv_readings, "epics_checks":epics_checks} )


def get_epics_checks(path, pvlist):
    epics_pv_list = []
    used_pv_obj={}
    epics_ojb = {
        "MCIS:BIAS0:VOL":"IOS:BIAS",
        "IOS:BIAS:VOL":"IOS:BIAS",
        "IOS:MB:MASSOVERQ2":"IOS:ISOTOPE",
        "ITW:BIAS:VOL":"IMS:BIAS",
        "ITE:BIAS:VOL":"IMS:BIAS",
        "CSB:HVBIAS:VOL":"CTIS:BIAS",
        "IMS:MQRATIO:MB1":"IMS:ISOTOPE",
        "IMS:MQRATIO:MB2":"IMS:ISOTOPE",
        "CSB:MB14:MASSOVERQ2":"CTIS:ISOTOPE",
        "HEBT:STRP5:MOTOR.LLS":"HEBT:CHARGE",
        "SEBT:STRP20A:MOTOR.LLS":"SEBT:CHARGE",
    };

    dtl_pv_list = ["ISAC1:DTL1:RDREADY", "ISAC1:DTL2:RDREADY","ISAC1:DTL3:RDREADY","ISAC1:DTL4:RDREADY","ISAC1:DTL5:RDREADY",]

    #Selecting which BIAS to use and loading object with relevent PV's
    for pv in epics_ojb:
        if epics_ojb[pv] in pvlist:
            if "BIAS" in epics_ojb[pv]:
                if "itw" in path and "ITW" in pv:
                    epics_pv_list.append(pv)
                    used_pv_obj[pv] = epics_ojb[pv]                    
                if "ite" in path and "ITE" in pv:
                    epics_pv_list.append(pv)
                    used_pv_obj[pv] = epics_ojb[pv]                       
                if "mcis" in path and "MCIS" in pv:
                    epics_pv_list.append(pv)
                    used_pv_obj[pv] = epics_ojb[pv]    
                if ("mws" in path or "sis" in path) and "IOS" in pv:
                    epics_pv_list.append(pv)
                    used_pv_obj[pv] = epics_ojb[pv]
                if "csb" in path and "CSB" in pv:
                    epics_pv_list.append(pv)
                    used_pv_obj[pv] = epics_ojb[pv]                     
            else:
                epics_pv_list.append(pv)
                used_pv_obj[pv] = epics_ojb[pv]

    epics_pv_readings = jaya.get_pvs(epics_pv_list) 
    dtl_pv_readings = jaya.get_pvs(dtl_pv_list)

    return {"epics_pvs_vals":epics_pv_readings,
            "beam_props_pvs":  used_pv_obj,
            "dtl_pv_readings": dtl_pv_readings,
            }


@mod_apex.route("/_get_posible_paths",methods = ["GET"])
def _get_posible_paths():
    rib_paths = get_paths('ariel') + get_paths('isac')
    paths = [ {'name': p, 'url': "url_for('change_path', path=p)"  }
                for p in rib_paths ]

    return jsonify({'paths': paths})


@mod_apex.route("/_get_path_info",methods = ["POST"])
def _get_path_info():
    path_pvs = ['IMS:PATH', 'CSB:PATH', 'IOS:PATH', 'ALTIS:PATH']
    path_dict = jaya.get_pvs(path_pvs)

    return_data = [path_dict, ]
    return jsonify(data[0])


def get_paths(facility):
	facility_dir = os.path.join(accBaseDir, facility, 'path')
	output = [ item.replace(".xml","") for item in os.listdir(facility_dir) ]
	return sorted(output)


@mod_apex.route("/landing",methods = ["GET"])
def landing_page():
    settings = {
    'title': 'Beam Selection',
    'app_name': 'Beam Selection',
    'app_url': url_for('apex.landing_page'),
    'gitlab_url': 'https://gitlab.triumf.ca/hla/apex'
    }

    tunex_url = ""
    current_url = url_for('apex.landing_page', _external=True)
    if 'devel' in current_url or 'localhost' in current_url:
        tunex_url = 'https://tunex.dev.kube.dmz.triumf.ca/restore'
    if 'beta' in current_url:
        tunex_url = 'https://tunex.triumf.ca/restore'

    link1 = {'text':'Help page', 'url': 'https://beta.hla.triumf.ca/snapshot/helppage'}
    link2 = {'text':'Search / Restore tune', 'url': tunex_url}
    dropdown_links = [link1, link2]
    navbar = [{'type':'dropdown', 'text':'Extra Tools','links':dropdown_links}]


    path_pvs = ['IMS:PATH', 'CSB:PATH', 'IOS:PATH', 'ALTIS:PATH']
    path_dict = jaya.get_pvs(path_pvs)

    rib_paths = get_paths('ariel') + get_paths('isac')
    paths = [ {'name': p, 'url': "url_for('change_path', path=p)"  }
                for p in rib_paths ]

    return render_template('beam_selection.html', header_settings=settings, 
        hla_access='true', beampaths=path_dict, paths=paths, navbar_left=navbar)